@@ -42,9 +42,9 @@
 
 
 startingVLSTAR <- function(y, exo = NULL, p = 1,
-                   m = 2, st = NULL, constant = TRUE,
-                   n.combi = NULL, ncores = 2,
-                   singlecgamma = FALSE){
+                           m = 2, st = NULL, constant = TRUE,
+                           n.combi = NULL, ncores = 2,
+                           singlecgamma = FALSE){
   y <- as.matrix(y)
   x <- exo
   if (anyNA(y))
@@ -97,116 +97,128 @@ startingVLSTAR <- function(y, exo = NULL, p = 1,
       x <- as.matrix(cbind(const,ylag))
     }
     ncolx <- ncol(x)
-  }  else{
-    x <- as.matrix(x)
+  } else{
+    if(!is.null(exo)){
+      x1a <- as.matrix(x[-c(1:p),])
+      x <- as.matrix(cbind(ylag,x1a))
+    }else{
+      x <- as.matrix(ylag)
+    }
   }
   nrowx <- nrow(x)
   st <- st[(1+p):length(st)]
   ncolx <- ncol(x)
   param.init <- list()
-  if(singlecgamma == TRUE){
-    param.init$gamma <- 1
+  # Starting values for c and gamma
+  if (singlecgamma == TRUE) {
+    param.init$gamma <- 2L
     param.init$c <- mean(y)
-  } else{
-    param.init$gamma <- rep(1L, ncoly)
+  } else {
+    param.init$gamma <- rep(2L, ncoly)
     param.init$c <- colMeans(y)
   }
+
   ny <- ifelse(singlecgamma == TRUE, 1, ncoly)
-  q <- ncol(x)-ncolylag
   COMBI <- list()
   GAMMA <- list()
   CJ <- list()
-  for(t in 1:(m-1)){
-      #Starting values for c and gamma
-      gamma <- matrix(ncol = ny, nrow = n.combi)
-      cj <- matrix(ncol = ny, nrow = n.combi)
-      gamma[1,] <- param.init$gamma
-      cj[1,] <- param.init$c
-      GAMMA[[t]] <- gamma*(1+rnorm(1, mean = 0, sd = 5))
-      CJ[[t]] <- cj*(1+rnorm(1, mean = 0, sd = 5))
-      #Grid for c and gamma
+  for (t in 1:(m - 1)) {
+    if (singlecgamma == TRUE) {
+      rangey = range(st)
+      gamma <- seq(from = 1L, to = 20, length.out = n.combi)
+      cj <- seq(from = rangey[1], to = rangey[2], length.out = n.combi)
+      GAMMA[[t]] <- gamma
+      CJ[[t]] <- cj
+      COMBI[[t]] <- expand.grid(CJ[[t]], GAMMA[[t]])
+    } else {
+      GAMMA[[t]] <- matrix(ncol = ny, nrow = n.combi)
+      CJ[[t]] <- matrix(ncol = ny, nrow = n.combi)
       rangey <- matrix(nrow = ny, ncol = 2)
       combi <- list()
-      for(j in 1:ny){
-        rangey[j,] <- range(y[,j])
-        CJ[[t]][2:n.combi,j] <- seq(from = rangey[j,1]*1.1, to = rangey[j,2], length.out = (n.combi-1))
-        GAMMA[[t]][2:n.combi, j] <- seq(from = 0L, to = 100L, length.out = (n.combi-1))
-        combi[[j]] <- expand.grid(CJ[[t]][,j], GAMMA[[t]][,j])
+      for (j in 1:ny) {
+        #rangey[j, ] <- range(y[, j])
+        rangey[j, ] <- range(st)
+        CJ[[t]][, j] <- seq(from = rangey[j, 1], to = rangey[j, 2], length.out = n.combi)*1/(1+exp(-t))
+        GAMMA[[t]][, j] <- seq(from = 1L, to = 20, length.out = n.combi)*(1+exp(-t))
+        combi[[j]] <- expand.grid(CJ[[t]][, j], GAMMA[[t]][, j])
       }
       COMBI[[t]] <- combi
-      }
+    }
+  }
 
-#NLS for each combination of c and gamma
+  #NLS for each combination of c and gamma
 
-message(paste('Searching optimal c and gamma among', n.combi*n.combi, 'combinations\n'))
-cl <- parallel::makeCluster(ncores)
-doSNOW::registerDoSNOW(cl)
-pb <- txtProgressBar(max = n.combi*n.combi, style = 3)
-progress <- function(n) setTxtProgressBar(pb, n)
-opts <- list(progress = progress)
-l = 1
-ssq <- foreach(l = 1:(n.combi*n.combi), .errorhandling='pass', .combine = rbind,.options.snow = opts) %dopar% {
+  message(paste('Searching optimal c and gamma among', n.combi*n.combi, 'combinations\n'))
+  cl <- parallel::makeCluster(ncores)
+  doSNOW::registerDoSNOW(cl)
+  pb <- txtProgressBar(max = n.combi*n.combi, style = 3)
+  progress <- function(n) setTxtProgressBar(pb, n)
+  opts <- list(progress = progress)
+  l = 1
+  ssq <- foreach(l = 1:(n.combi*n.combi), .errorhandling='pass', .combine = rbind,.options.snow = opts) %dopar% {
     Sys.sleep(0.1)
-      glog <- matrix(ncol=ny, nrow = nrowy)
-      GT <- list()
-      Gtilde <- list()
-      kro <- list()
-      for (i in 1:nrowx){
-        for(t in 1:(m-1)){
-          for (j in 1 : ny){
+    glog <- matrix(ncol=ny, nrow = nrowy)
+    GT <- list()
+    Gtilde <- list()
+    kro <- list()
+    for (i in 1:nrowx){
+      for(t in 1:(m-1)){
+        if(singlecgamma == TRUE){
+          glog[i,] <- (1+exp(-COMBI[[t]][l,2]*(st[i]-COMBI[[t]][l,1])))^(-1)
+          Gt <- diag(rep(glog[i,1], ncoly))
+        }else{
+          for(j in 1:ny){
             glog[i,j] <- (1+exp(-COMBI[[t]][[j]][l,2]*(st[i]-COMBI[[t]][[j]][l,1])))^(-1)
           }
-          if(singlecgamma == TRUE){
-            Gt <- diag(rep(glog[i,1], ncoly))
-          }else{
-            Gt <- diag(glog[i,])
-          }
-          GT[[t]] <- Gt
+          Gt <- diag(glog[i,])
         }
-        Gtilde[[i]] <- t(cbind(In, do.call(cbind,GT)))
-        kro[[i]] <- kronecker(Gtilde[[i]], x[i,])
+        GT[[t]] <- Gt
       }
-      M <- t(do.call("cbind", kro))
-      Y <- matrixcalc::vec(t(y))
-      Bhat <- MASS::ginv(t(M)%*%M)%*%t(M)%*%Y
-      BB <- ks::invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
-      resi <- list()
-      Ehat <- matrix(NA, ncol = ncoly, nrow = nrowy)
-      for (o in 1:nrowx){
-        resi[[o]] <- y[o, ] - t(Gtilde[[o]])%*%t(BB)%*%x[o,]
-      }
-      Ehat <- t(do.call("cbind", resi))
-      SSQ <- colSums(Ehat^2)
-      return(SSQ)
-}
-close(pb)
-stopCluster(cl)
-#c and gamma minimazing the sum of squared residuals for each equation
+      Gtilde[[i]] <- t(cbind(In, do.call(cbind,GT)))
+      kro[[i]] <- kronecker(Gtilde[[i]], x[i,])
+    }
+    M <- t(do.call("cbind", kro))
+    Y <- matrixcalc::vec(t(y))
+    Bhat <- MASS::ginv(t(M)%*%M)%*%t(M)%*%Y
+    BB <- ks::invvec(Bhat, ncol = (m*ncoly), nrow = nrow(Bhat)/(m*ncoly))
+    Ehat <- matrix(NA, ncol = ncoly, nrow = nrowy)
+    for (o in 1:nrowx){
+      Ehat[o,] <- t(y[o, ] - t(Gtilde[[o]])%*%t(BB)%*%x[o,])
+    }
+    SSQ <- colSums(Ehat^2)
+    return(SSQ)
+  }
+  close(pb)
+  stopCluster(cl)
+  #c and gamma minimazing the sum of squared residuals for each equation
+  if(singlecgamma == TRUE){
+    cgamma <- vector(length = 2L)
+  }else{
+    cgamma <- matrix(nrow = ny, ncol = 2)
+  }
 
-cgamma <- matrix(nrow = ny, ncol = 2)
-CGAMMA <- list()
-for (t in 1:(m-1)){
-  if(singlecgamma == T){
-    cgamma[j,] <-  as.matrix(COMBI[[t]][[j]][which.min(rowSums(ssq)),])
+  CGAMMA <- list()
+  for (t in 1:(m-1)){
+    if(singlecgamma == T){
+      cgamma <-  as.matrix(COMBI[[t]][which.min(rowSums(ssq)),])
     } else{
       for(j in 1:ny){
         cgamma[j,] <- as.matrix(COMBI[[t]][[j]][which.min(ssq[,j]),])
+        #cgamma[j,] <- as.matrix(COMBI[[t]][[j]][which.min(rowSums(ssq)),])
       }
-      }
-  CGAMMA[[t]] <- cgamma
+    }
+    CGAMMA[[t]] <- cgamma
   }
 
 
-#Definition of c0 gamma0
-PARAM <- list()
-for (t in 1:(m-1)){
-  cj <- as.matrix(CGAMMA[[t]][,1])
-  gamma <- as.matrix(CGAMMA[[t]][,2])
-  PARAM[[t]] <- cbind(gamma, cj)
+  #Definition of c0 gamma0
+  PARAM <- list()
+  for (t in 1:(m-1)){
+    cj <- as.matrix(CGAMMA[[t]][,1])
+    gamma <- as.matrix(CGAMMA[[t]][,2])
+    PARAM[[t]] <- cbind(gamma, cj)
   }
-results <- PARAM
-class(results) = 'startingVLSTAR'
-return(results)
+  results <- PARAM
+  class(results) = 'startingVLSTAR'
+  return(results)
 }
-
-
